/******************************************************************
 Created with PROGRAMINO IDE for Arduino - 30.01.2022
 Project      : SCHRITTMOTOR MIT jOYSTICK STEUERN
 Libraries    : AccelStepper.h / TM1637Display.h / EEPROM.h
 Author       : ahmedazouz => Snippets ""Labyrinth mit Servomotoren""
                starthardware => Snippets "Zeitmessung"
                BEASTIDREES62 => Snippets "Stepper mit Joystick steuern"
                Wolfgang Keller => - Verknüpfung der Programmteile und Anpassungen an Steppermotoren
                                   - einfügen Highscore- Ausgabe 4 Digit Display
                                   - einfügen laufende Zeitausgabe und Endzeit über 4 Digit Display
                                   - Auslesen Lichtschranken über IC 74HC132N
 Beschreibung : Die Stepper werden mit dem Joystick rechts/links gesteuert 
                Pin-Belegung für Arduino >Nano
 Version 03_7   : läuft 
                - Geschwindigkeit durch Joystick regelbar
                - in x und y-Richtung mit Begrenzung
                - Zeitanzeige funktioniert
                - Lichtschranke funktioniert
                - Joystick dreht in alle Richtungen richtig
                - Highscore und EEPROM schreiben 
                - Highscore wird z.Zt. noch im setup auf null gesetzt, muß später entfernt werden
                
******************************************************************/
#include <AccelStepper.h>
#include <TM1637Display.h>
#include <EEPROM.h>
#define CLK 3               // für Anzeige
#define DIO 2                 // für Anzeige
#define numberOfSeconds(xyz) ((xyz / 1000) % 60)
#define numberOfMinutes(xyz) (((xyz / 1000) / 60) % 60)
TM1637Display display(CLK, DIO);
// Motor_y pin definitions:
#define motoryPin1 6 // IN1 on the ULN2003 driver
#define motoryPin2 7 // IN2 on the ULN2003 driver
#define motoryPin3 8 // IN3 on the ULN2003 driver
#define motoryPin4 9 // IN4 on the ULN2003 driver
// Motor_x pin definitions:
#define motorxPin1 10 // IN1 on the ULN2003 driver
#define motorxPin2 11 // IN2 on the ULN2003 driver
#define motorxPin3 12 // IN3 on the ULN2003 driver
#define motorxPin4 13 // IN4 on the ULN2003 driver
// Define the AccelStepper interface type; 4 wire motor in half step mode:
#define MotorInterfaceType 8
// Define a stepper and the pins it will use
AccelStepper stepper_y = AccelStepper(MotorInterfaceType, motoryPin1, motoryPin3,
motoryPin2, motoryPin4);
AccelStepper stepper_x = AccelStepper(MotorInterfaceType, motorxPin1, motorxPin3,
motorxPin2, motorxPin4);
 
// joystick pot output is connected to Arduino A0
#define joystick_y  A0
#define joystick_x  A1
int val_y = 0;
int val_x = 0;

int range = 24;
int center = range/2;         // resting position value
int threshold = range/12;      // resting threshold
unsigned long int hscore = 0;
const int beginn = 4  ;             // Pin 4 Start 
const int ende = 5 ;                // Pin 5 Ende 
int anzeigeDauer_endeZeit = 5000;   // millisekunden
int stateStart;
int programState = 0;
unsigned long startZeit = 0;
unsigned long myTime;
long zeitAngehalten;    
unsigned long myTimer1=0; 
unsigned long int endZeit = 0;
unsigned long pauseEnde = 0;
unsigned long pauseHscore = 0 ; 
unsigned long int umrech = 1000;
unsigned long int eprom_score = 0;

void setup()
{
  Serial.begin(9600);  
  pinMode(beginn,INPUT);         // von Lichtsachranke
  pinMode(ende,INPUT);           // von Lichtschranke 
  display.setBrightness(1);
  //EEPROM.Write nach der Erstinstallation entfernen, damit der hscore nicht nach jedem 
  //Ein-Ausschalten verschwindet
  //EEPROM.write(0,hscore) ;       // Eprom = 0 setzen
}
 
void loop()
{

  stepper_y.setMaxSpeed(500);
  stepper_y.setAcceleration(100);
  stepper_x.setMaxSpeed(500);
  stepper_x.setAcceleration(200);
  // read analog value from the potentiometer
  val_x = readAxis(joystick_x);   // XAxlePin = A0
  val_y = readAxis(joystick_y);   // XAxlePin = A1
  if  ((val_y == 0) && (val_x == 0)) 
  {
    stepper_y.stop();
    stepper_x.stop();
  }
 
  else
  {
    // move the motory in the links direction
    while ((val_y > 0) && (stepper_y.currentPosition() >=-140)) // vorne runter
    {
      // map the speed between 0 and 300 rpm
      int speed_ = map(val_y, 0, 24, 0, 300);
      // set motor speed
      stepper_y.setSpeed(-speed_); 
      stepper_y.runSpeed(); 
      // move the motor (1 step) every loop
        val_y = readAxis(joystick_y);
    }
    // move the motory in the rechts direction
    while ((val_y < 0 ) && (stepper_y.currentPosition() <= 140)) // hinten runter
    {
      // map the speed between 0 and 300 rpm
      int speed_ = map(val_y, 0, -24, 0, 300);
      // set motor speed
      stepper_y.setSpeed(speed_);
      stepper_y.runSpeed();
      // move the motor (1 step) every loop
        val_y = readAxis(joystick_y);
    }
    // move the motorx in the links direction
    while ((val_x > 0) && (stepper_x.currentPosition() >= -140))  // links
    {
      // map the speed between 0 and 300 rpm
      int speed_ = map(val_x, 0, 24, 0, 300);
      // set motor speed
      stepper_x.setSpeed(-speed_); 
      stepper_x.runSpeed(); 
      // move the motor (1 step) every loop
      val_x = readAxis(joystick_x); 
    }
 // move the motorx in the rechts direction
    while ((val_x < 0) && (stepper_x.currentPosition() <= 140)) // rechts
    {
      // map the speed between 0 and 300 rpm
      int speed_ = map(val_x, 0,-24, 0, 300);
      // set motor speed
      stepper_x.setSpeed(speed_);
      stepper_x.runSpeed();
      // move the motor (1 step) every loop
      val_x = readAxis(joystick_x); 

    }    

  }
  
zeitmessung();
}

//#######################################################################
void zeitmessung()
{
  stateStart = digitalRead(beginn);
  zeitAngehalten = digitalRead(ende);

  switch (programState) {
    case 0: // gestoppt, Anzeige auf Null gestellt, warten auf startpin
      startZeit = 0;
      display.showNumberDecEx(0, 0b01000000, true, 2, 2);
      display.showNumberDecEx(0, 0b01000000, true, 2, 0);
      // start
      if (stateStart == LOW) {   // Lichtschranke unterbrochen
        startZeit = millis();
        programState = 1;
      }
      break;
    case 1: // gestartet, Anzeige wird hochgezält, warten auf endePin
      showTime(millis());
      // stop
      if (zeitAngehalten == LOW)  {   // Lichtschranke unterbrochen
        
        programState = 3;
      }
      break;
    case 3: // gestoppt,  End-Anzeige
      showTime(millis());
        myTimer1 = millis();
        pauseEnde = anzeigeDauer_endeZeit + myTimer1 ;
        while (millis() < pauseEnde ) // Anzeigedauer Endezeit für 5 Sekunden
        { 
        Serial.print("case 3 millis = ");
        Serial.println(millis());
        Serial.println();
        }  
        programState = 4;
      break;
    case 4: // highscore-Anzeige
      highscore();
        myTimer1 = millis(); 
        pauseHscore = anzeigeDauer_endeZeit + myTimer1;
        while (millis() < pauseHscore ) // Anzeigedauer aktueller Highscore
        {
        Serial.print("case 4 millis = ");
        Serial.println(millis());
        Serial.println();       
        }   
        programState = 0;
      break;      
    }
}
//############### High Score  ######################################
void highscore()
{
endZeit = myTime/1000;                               // im EPROM nur Sekunden gespeichert, max 255 oder 4 min 25 Sek
hscore = EEPROM.read(0);  

    if (hscore == 0)                               // beim ersten Durchlauf ist hscore = 0
    {
      EEPROM.write(0, endZeit);                        // Endzeit in sekunden als hscore in EPROM schreiben   

    }
    else if ( endZeit < hscore)
    {
      EEPROM.write(0,endZeit) ;                        // Endzeit in sekunden als hscore in EPROM schreiben   
    }
  eprom_score = EEPROM.read(0);                       // für Anzeige in millisekunden numrechnen
  hscore = eprom_score * umrech;
   
  int seconds = numberOfSeconds(hscore);
  int minutes = numberOfMinutes(hscore);

  display.showNumberDecEx(seconds, 0, true, 2, 2);              // Anzeige hscore Sekunden
  if (seconds % 2 == 0) {
    display.showNumberDecEx(minutes, 0b01000000, true, 2, 0);   // Anzeige hscore Minuten
  } else {
    display.showNumberDecEx(minutes, 0, true, 2, 0);            // Anzeige hscore Minuten
  }

}
//#######################################################
void showTime(long theTime) {    // über den Aufruf showTime(millis()) werden die aktuellen millis an theTime übergeben

  myTime = theTime - startZeit;
  int seconds = numberOfSeconds(myTime);
  int minutes = numberOfMinutes(myTime);

  display.showNumberDecEx(seconds, 0, true, 2, 2);
  if (seconds % 2 == 0) {
    display.showNumberDecEx(minutes, 0b01000000, true, 2, 0);
  } else {
    display.showNumberDecEx(minutes, 0, true, 2, 0);
  }

}
//#########################################
int readAxis(int thisAxis) {
// read the analog input:
int reading = analogRead(thisAxis);
// map the reading from the analog input range to the output range:
reading = map(reading, 0, 1023, 0, range);    // der Wert von "reading" liegt zwischen 0...1023 und soll
                                              // auf den Bereich 0....range(24) umgerechnet werden
// if the output reading is outside from the
// rest position threshold, use it:
int distance = reading - center;              // center = 
if (abs(distance) < threshold) {              // threshold = 3
distance = 0;
}
// return the distance for this axis:
return distance;
}